import 'package:dio/dio.dart';
import 'package:flutter/material.dart';
import 'package:lori_sdk/data/database/interfaces/userDao.dart';
import 'package:lori_sdk/data/lori_api.dart';



class SalesBuyerApi {
  SalesBuyerApi({@required Dio requester, UserDao userDao}) : this._requester = requester, this._userDao = userDao;

  Dio _requester;
  UserDao _userDao;



  static const USER = 'sales/buyer/buyers/';

  String parseResponseError(dynamic response) {
    if (response == null) {
      return null;
    } else if (response is Map<String, dynamic>) {
      Map<String, dynamic> map = response;
      if (map.containsKey('error')) {
        String errorDescription = map['error'];
        switch (errorDescription) {
          case 'invalid_grant':
            return 'Usuário e/ou senha inválido';
        }
      } else if (map.containsKey('message')) {
        return map['message'];
      } else if (map.containsKey('Message')) {
        return map['Message'];
      }
    }
    return null;
  }


  Future<ApiCallback<void>> updateUser(String fullName, String cpf, String phoneNumber,{String dateOfBirth, int gender, String password, String newPassword}) async {
    var loggedUser = await _userDao.get();
    try {
      Map body = {
        'id': loggedUser.id,
        'fullName': fullName,
        'cpf': cpf,
        'phoneNumber': phoneNumber,
        'password' : password,
      };
      if (dateOfBirth != null) {
        body['dateOfBirth'] = dateOfBirth;
      }
      if (gender != null) {
        body['gender'] = gender;
      }
      if (newPassword != null) {
        body['newpassword'] = newPassword;
      }
      await _requester.put("$USER${loggedUser.id}", data: body, options: Options(responseType: ResponseType.json));
      // Parse the user

      // Update fields

      loggedUser.fullName = fullName;
      loggedUser.cpf = cpf;
      loggedUser.phoneNumber = phoneNumber;
      loggedUser.dateOfBirth = dateOfBirth != null ? dateOfBirth : loggedUser.dateOfBirth;
      loggedUser.gender = gender != null ? gender : loggedUser.gender;
      await _userDao.insertOrUpdate(loggedUser);
      // Callback invocation
      return ApiCallback(isSuccessful: true);
    } on DioError catch (e) {
      return ApiCallback(isSuccessful: false, resultErrorMessage: parseResponseError(e.response != null ? e.response.data : null));
    }
  }
}