import 'dart:async';
import 'dart:convert';

import 'package:lori_sdk/data/database/interfaces/notificationDao.dart';
import 'package:lori_sdk/data/models/notification.dart';
import 'package:lori_sdk/data/models/singleton/notificationSingleton.dart';
import 'package:shared_preferences/shared_preferences.dart';

const int kVersion1 = 1;

class WebNotificationDao implements NotificationDao {
  static const _NOTIFICATIONS_KEY = 'x_notifications';

  SharedPreferences db;

  Future open() async {
    if (db == null) {
      db = await SharedPreferences.getInstance();
    }
  }

  Future<void> insertOrUpdate(Notification notification) async {
    // Get by text
    if (notification.id == null || notification.id == 0) {
      var notif = await getByText(notification.title, notification.message);
      if (notif == null) {
        return insert(notification);
      } else {
        notif.title = notification.title;
        notif.message = notification.message;
        return updateByText(notif);
      }
    } else {
      // Get by id
      var notif = await getById(notification.id);
      if (notif == null) {
        return insert(notification);
      } else {
        return update(notification);
      }
    }
  }

  Future<void> insert(Notification notification) async {
    await open();
    List<Notification> list = await _all();
    list.add(notification);
    await db.setString(_NOTIFICATIONS_KEY,
        jsonEncode(list.map((item) => item.toJson()).toList()));
    return NotificationSingleton().notificationNumberChanged(list.length);
  }

  Future<void> update(Notification notification) async {
    await open();
    List<Notification> list = await _all();
    for (int i = 0; i < list.length; i++) {
      if (list[i].id == notification.id) {
        list[i] = notification;
        break;
      }
    }
    await db.setString(_NOTIFICATIONS_KEY,
        jsonEncode(list.map((item) => item.toJson()).toList()));
    return NotificationSingleton().notificationNumberChanged(list.length);
  }

  Future<void> updateByText(Notification notification) async {
    await open();
    List<Notification> list = await _all();
    for (int i = 0; i < list.length; i++) {
      if (list[i].message == notification.message) {
        list[i] = notification;
        break;
      }
    }
    await db.setString(_NOTIFICATIONS_KEY,
        jsonEncode(list.map((item) => item.toJson()).toList()));
    return NotificationSingleton().notificationNumberChanged(list.length);
  }

  Future<List<Notification>> get(bool isRead) async {
    await open();
    List<Notification> list = await _all();
    List<Notification> read = [];
    for (var item in list) {
      if (item.isRead == isRead) {
        read.add(item);
      }
    }
    return read;
  }

  Future<Notification> getByText(String title, String body) async {
    await open();
    List<Notification> list = await _all();
    for (var item in list) {
      if (item.title == title && item.message == body) {
        return item;
      }
    }
    return null;
  }

  Future<Notification> getById(int id) async {
    await open();
    List<Notification> list = await _all();
    for (var item in list) {
      if (item.id == id) {
        return item;
      }
    }
    return null;
  }

  Future<void> deleteAll() async {
    await open();
    await db.setString(_NOTIFICATIONS_KEY, null);
    return;
  }

  Future<List<Notification>> _all() async {
    await open();
    String notificationsInJson = db.getString(_NOTIFICATIONS_KEY);
    List<Notification> responses = [];
    if (notificationsInJson != null) {
      List<Map> responsesMap = List.from(jsonDecode(notificationsInJson));
      for (var map in responsesMap) {
        responses.add(Notification.fromJson(map));
      }
    }
    return responses;
  }
}

NotificationDao getNotificationDao() => WebNotificationDao();
